
/***************************************************************************
 * common.h --  This file holds all those functions and classes that have  *
 * been copied verbatim from nmap's source code                            *
 *                                                                         *
 ***********************IMPORTANT NMAP LICENSE TERMS************************
 *                                                                         *
 * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
 * also a registered trademark of Insecure.Com LLC.  This program is free  *
 * software; you may redistribute and/or modify it under the terms of the  *
 * GNU General Public License as published by the Free Software            *
 * Foundation; Version 2 with the clarifications and exceptions described  *
 * below.  This guarantees your right to use, modify, and redistribute     *
 * this software under certain conditions.  If you wish to embed Nmap      *
 * technology into proprietary software, we sell alternative licenses      *
 * (contact sales@insecure.com).  Dozens of software vendors already       *
 * license Nmap technology such as host discovery, port scanning, OS       *
 * detection, and version detection.                                       *
 *                                                                         *
 * Note that the GPL places important restrictions on "derived works", yet *
 * it does not provide a detailed definition of that term.  To avoid       *
 * misunderstandings, we consider an application to constitute a           *
 * "derivative work" for the purpose of this license if it does any of the *
 * following:                                                              *
 * o Integrates source code from Nmap                                      *
 * o Reads or includes Nmap copyrighted data files, such as                *
 *   nmap-os-db or nmap-service-probes.                                    *
 * o Executes Nmap and parses the results (as opposed to typical shell or  *
 *   execution-menu apps, which simply display raw Nmap output and so are  *
 *   not derivative works.)                                                *
 * o Integrates/includes/aggregates Nmap into a proprietary executable     *
 *   installer, such as those produced by InstallShield.                   *
 * o Links to a library or executes a program that does any of the above   *
 *                                                                         *
 * The term "Nmap" should be taken to also include any portions or derived *
 * works of Nmap.  This list is not exclusive, but is meant to clarify our *
 * interpretation of derived works with some common examples.  Our         *
 * interpretation applies only to Nmap--we don't speak for other people's  *
 * GPL works.                                                              *
 *                                                                         *
 * If you have any questions about the GPL licensing restrictions on using *
 * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
 * we also offer alternative license to integrate Nmap into proprietary    *
 * applications and appliances.  These contracts have been sold to dozens  *
 * of software vendors, and generally include a perpetual license as well  *
 * as providing for priority support and updates as well as helping to     *
 * fund the continued development of Nmap technology.  Please email        *
 * sales@insecure.com for further information.                             *
 *                                                                         *
 * As a special exception to the GPL terms, Insecure.Com LLC grants        *
 * permission to link the code of this program with any version of the     *
 * OpenSSL library which is distributed under a license identical to that  *
 * listed in the included COPYING.OpenSSL file, and distribute linked      *
 * combinations including the two. You must obey the GNU GPL in all        *
 * respects for all of the code used other than OpenSSL.  If you modify    *
 * this file, you may extend this exception to your version of the file,   *
 * but you are not obligated to do so.                                     *
 *                                                                         *
 * If you received these files with a written license agreement or         *
 * contract stating terms other than the terms above, then that            *
 * alternative license agreement takes precedence over these comments.     *
 *                                                                         *
 * Source is provided to this software because we believe users have a     *
 * right to know exactly what a program is going to do before they run it. *
 * This also allows you to audit the software for security holes (none     *
 * have been found so far).                                                *
 *                                                                         *
 * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 * and add new features.  You are highly encouraged to send your changes   *
 * to nmap-dev@insecure.org for possible incorporation into the main       *
 * distribution.  By sending these changes to Fyodor or one of the         *
 * Insecure.Org development mailing lists, it is assumed that you are      *
 * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
 * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
 * will always be available Open Source, but this is important because the *
 * inability to relicense code has caused devastating problems for other   *
 * Free Software projects (such as KDE and NASM).  We also occasionally    *
 * relicense the code to third parties as discussed above.  If you wish to *
 * specify special license conditions of your contributions, just say so   *
 * when you send them.                                                     *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License v2.0 for more details at                         *
 * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
 * included with Nmap.                                                     *
 *                                                                         *
 ***************************************************************************
 * The code in this file was originally written by Luis MartinGarcia       *
 * during Google Summer of Code programme, 2009.                           *
 *                                                                         *
 ***************************************************************************/

/* THIS FILE HAS ALL THE FUNCTIONS THAT I'VE COPIED VERBATIM FROM NMAP'S
 * SOURCE CODE. */


/* NOTE: Many of theese functions had the "static" qualifier. I don't know
 * why but that causes compiler error so I've removed it from many of the
 * functions. Take this into account when moving these functions to nbase
 * or wherever. */



#ifndef COMMON_H
#define COMMON_H 1

#include "pcap.h"
#include "dnet.h"
#include "nping.h"


/* Solaris does not seem to define u_intXX_t but only uintXX_t */
#ifdef SOLARIS
typedef uint8_t u_int8_t;
#endif

/* For Solaris ioctl() flags: SIOCGIFCONF, SIOCGIFFLAGS and SIOCGIFNETMASK*/
#ifdef SOLARIS
#include <sys/sockio.h> 
#endif

/* TODO: Included by me, may not be portable */
#ifndef WIN32
#include <net/if.h>
#include <sys/ioctl.h>
#endif
/*****************************************************************************
  * STUFF FROM tcpip.h
  ****************************************************************************/






 struct ip
  {
#if WORDS_BIGENDIAN
    u_int8_t ip_v:4;                    /* version */
    u_int8_t ip_hl:4;                   /* header length */
#else
    u_int8_t ip_hl:4;                   /* header length */
    u_int8_t ip_v:4;                    /* version */
#endif
    u_int8_t ip_tos;                    /* type of service */
    u_short ip_len;                     /* total length */
    u_short ip_id;                      /* identification */
    u_short ip_off;                     /* fragment offset field */
#define IP_RF 0x8000                    /* reserved fragment flag */
#define IP_DF 0x4000                    /* dont fragment flag */
#define IP_MF 0x2000                    /* more fragments flag */
#define IP_OFFMASK 0x1fff               /* mask for fragmenting bits */
    u_int8_t ip_ttl;                    /* time to live */
    u_int8_t ip_p;                      /* protocol */
    u_short ip_sum;                     /* checksum */
    struct in_addr ip_src, ip_dst;      /* source and dest address */
  };


  struct eth_nfo {
  char srcmac[6];
  char dstmac[6];
  eth_t *ethsd; // Optional, but improves performance.  Set to NULL if unavail
  char devname[16]; // Only needed if ethsd is NULL.
};


typedef enum { devt_ethernet, devt_loopback, devt_p2p, devt_other  } devtype;

/* Relevant (to Nmap) information about an interface */
struct interface_info {
  char devname[16];
  char devfullname[16]; /* can include alias info, such as eth0:2. */
  struct sockaddr_storage addr;
  u16 netmask_bits; /* CIDR-style.  So 24 means class C (255.255.255.0)*/
  devtype device_type; /* devt_ethernet, devt_loopback, devt_p2p, devt_other */
  bool device_up; /* True if the device is up (enabled) */
  u8 mac[6]; /* Interface MAC address if device_type is devt_ethernet */
};

struct route_nfo {
  struct interface_info ii;

/* true if the target is directly connected on the network (no routing
   required). */
  bool direct_connect;

/* This is the source address that should be used by the packets.  It
   may be different than ii.addr if you are using localhost interface
   to scan the IP of another interface on the machine */
  struct sockaddr_storage srcaddr;

  /* If direct_connect is 0, this is filled in with the next hop
     required to route to the target */
  struct sockaddr_storage nexthop;
};

struct sys_route {
  struct interface_info *device;
  u32 dest;
  u32 netmask;
  struct in_addr gw; /* gateway - 0 if none */
};


/*****************************************************************************
  * STUFF FROM tcpip.cc
  ****************************************************************************/

int ipaddr2devname(char *dev, const struct in_addr *addr);
int devname2ipaddr(char *dev, struct in_addr *addr);

// Definition of struct dnet_collector_route_nfo
//static int nmaskcmp(const void *a, const void *b);
struct interface_info *getInterfaceByName(char *iname);

int resolve(char *hostname, struct sockaddr_storage *ss, size_t *sslen,int pf);
unsigned short in_cksum(u16 *ptr,int nbytes);
int parse_ip_options(char *txt, u8 *data, int datalen, int* firsthopoff, int* lasthopoff);
//static inline char* STRAPP(const char *fmt, ...);
char* print_ip_options(u8* ipopt, int ipoptlen);
unsigned short magic_tcpudp_cksum(const struct in_addr *src, const struct in_addr *dst, u8 proto, u16 len, const void *hstart);
pcap_t *my_pcap_open_live(const char *device, int snaplen, int promisc, int to_ms) ;
bool pcap_selectable_fd_valid();
void tcppacketoptinfo(u8 *optp, int len, char *result, int bufsize);
char *ll2shortascii(unsigned long long bytes, char *buf, int buflen);
void sethdrinclude(int sd);
void set_ipoptions(int sd, void *opts, size_t optslen);
void set_ttl(int sd, int ttl);
eth_t *eth_open_cached(const char *device) ;
void eth_close_cached();
int send_frag_ip_packet(int sd, struct eth_nfo *eth, u8 *packet, unsigned int packetlen, unsigned int mtu);
//static struct sys_route *getsysroutes_proc(FILE *routefp, int *howmany);
//static int collect_dnet_routes(const struct route_entry *entry, void *arg);
//static struct dnet_collector_route_nfo *sysroutes_dnet_find_interfaces(struct dnet_collector_route_nfo *dcrn);
//static struct sys_route *getsysroutes_dnet(int *howmany);
struct sys_route *getsysroutes(int *howmany);
struct interface_info *getinterfaces(int *howmany);

#ifdef WIN32
bool DnetName2PcapName(const char *dnetdev, char *pcapdev, int pcapdevlen);
static int collect_dnet_interfaces(const struct intf_entry *entry, void *arg);
#endif

int send_ip_packet_eth(struct eth_nfo *eth, u8 *packet, unsigned int packetlen);
int send_ip_packet_sd(int sd, u8 *packet, unsigned int packetlen);

#define ARPCACHE_GET 1  /* WARNING: Defines taken from common.cc*/
#define ARPCACHE_SET 2
bool NmapArpCache(int command, struct sockaddr_storage *ss, u8 *mac) ;
int pcap_select(pcap_t *p, long usecs);
int pcap_select(pcap_t *p, struct timeval *timeout);
int my_pcap_get_selectable_fd(pcap_t *p);

/*****************************************************************************
  * STUFF FROM utils.cc
  ****************************************************************************/
int optcmp(const char *a, const char *b);
int parse_ip_options(char *txt, u8 *data, int datalen, int* firsthopoff, int* lasthopoff);
char* print_ip_options(u8* ipopt, int ipoptlen);

				
				
/*****************************************************************************
  * STUFF FROM utils.h
  ****************************************************************************/  				




/*****************************************************************************
  * STUFF FROM nmap.cc
  ****************************************************************************/
int ip_is_reserved(struct in_addr *ip);
int fileexistsandisreadable(const char *pathname);









#endif /* COMMON_H */



